<?php

// don't load directly
if (!defined('ABSPATH'))
    exit;

/*
 * FightBack API Class
 */
class FightBack_API
{
    public function __construct()
    {
        //add plugin page to admin panel
        add_action('admin_menu', array(&$this, 'create_admin_menu'));

        //add a POST listener
        add_action('init', array(&$this, 'post_listener'));

        //hourly cron jobs. wp_schedule_event() doesn't work properly
        add_action('init', array(&$this, 'check_version'));

        // hooks to initialize inspection
        add_action('delete_category', array(&$this, 'check_cat_qty'));
        add_action('switch_theme'/* after_setup_theme */, array(&$this, 'theme_change'));
    }

    public function create_admin_menu()
    {
        add_menu_page('fightBack', 'FightBack', 'manage_options', 'fightback', array(&$this, 'fightback_settings_page_handler'));
    }

    public function fightback_settings_page_handler()
    {
        if ($_POST)
        {
            seofb_sanitize_post();

            if (isset($_POST['seofb_connect']))
            {
	// connect handler
	$fightback_code = isset($_POST['fightback_code']) ? $_POST['fightback_code'] : get_option('fightback_code');

	if ($fightback_code != '')
	{
	    //check if there are less than MIN_CATS_QTY=3 post categories in blog
	    if (count($this->get_categories_all()) < MIN_CATS_QTY)
	        $errors[] = 'More than ' . MIN_CATS_QTY . ' post categories must be there in your blog. Only ' . count($this->get_categories_all()) . ' of them found.';
	    else
	    {
	        //send request to fightback server
	        $connect_res = $this->_connect_request($fightback_code);

	        if (FALSE === $connect_res)
	        {
	            $errors[] = 'Couldn\'t connect to the remote server.';
	        }

	        // set message
	        switch ($connect_res['status'])
	        {
	            case 'success':

		$this->_connect($fightback_code);
		$success = $connect_res['message'];
		break;

	            case 'error':
		$errors[] = $connect_res['message'];
		if (isset($connect_res['data']))
		    $errors = array_merge($errors, $connect_res['data']);

	            default:
		break;
	        }
	    }
	}
	else
	    $errors[] = 'Empty FightBack Code';
            }
            elseif (isset($_POST['seofb_disconnect']))
            {
	// send the disconnect request
	$disconnect_res = $this->_disconnect_request(get_option('fightback_code'), 'Disconnected manually by blog admin.');

	if (FALSE === $disconnect_res && !isset($_POST['seofb_clear_code']))
	    $errors[] = 'Couldn\'t connect to the remote server.';
	else
	{
	    switch ($disconnect_res['status'])
	    {
	        case 'error':

	            $this->_disconnect('Disconnected manually by blog admin.', @$_POST['seofb_clear_code']);

	            //$errors[] = $disconnect_res['message'];
	            break;

	        default:
	            break;
	    }
	}
            }
            elseif (isset($_POST['seofb_settings']))
            {
	// settings form handler
	$settings = $_POST['seofb'];

	// save categories
	if (!isset($settings['category']) || in_array('all', $settings['category']))
	    update_option('fightback_categories', NULL);
	else
	    update_option('fightback_categories', $settings['category']); //don't need to serialize - update_option() makes it automatically

        // save authors
	if (!isset($settings['author']) || in_array('all', $settings['author']))
	    update_option('fightback_authors', NULL);
	else
	    update_option('fightback_authors', $settings['author']); //don't need to serialize - update_option() makes it automatically

	$success = 'Settings saved.';
            }
        }
        // get current categories for FBN posting
        $cur_categories = get_option('fightback_categories'); //don't need to unserialize - get_option() makes it automatically
        // get all wp blog categories
        $seofb_all_categories = $this->get_categories_all();
        
        $cur_authors = get_option('fightback_authors');
        $seofb_all_authors = get_users(array('who' => 'authors'));

        // load view
        require_once (SEOFB_VIEWS_DIR . 'admin_settings_page.php');
    }

    /**
     * POST listener is used to detect a POST requests targeted to this plugin
     */
    public function post_listener()
    {
        /**
         * @todo: delete in next version (it is needed just after updating to the 2.0 version)
         */
        if (get_option('fightback_code') && !get_option('fightback_status'))
            update_option('fightback_status', array('status' => 'disconnected', 'date' => date('Y-m-d H:i'), 'reason' => 'Reconnect to FBN is needed'));

        //check on any SEOFB flag
        if (!isset($_POST['seofb_action']))
        {
            return;
        }

        seofb_sanitize_post();

        seofb_log('Fightback request is received: ' . print_r($_POST, TRUE) . 'with files: ' . print_r($_FILES, TRUE));

        switch ($_POST['seofb_action'])
        {
            //post from the fightback is received
            case 'post_from_fightback':
	//validate a WP API code
	if (!isset($_POST['wp_api_code']) || $_POST['wp_api_code'] != get_option('fightback_code'))
	{
	    echo 'error|Incorrect wp_api_code.';
	    seofb_log_die('Incorrect wp_api_code in the request.');
	}
	$this->_receive_fightback_post();
	break;

            //request to post deletion
            case 'delete_fightback_post':
	//validate a WP Post Verification code
	if (!isset($_POST['wp_post_verify_code']) || $_POST['wp_post_verify_code'] != get_post_meta($_POST['wp_post_id'], 'fightback_verify_code', TRUE))
	{
	    echo 'error|Incorrect fightback_verify_code.';
	    seofb_log_die('Incorrect fightback_verify_code in the request.');
	}
	$this->_delete_fightback_post();
	break;

            //connect to FBN
            case 'connect':
	$this->_connect();
	break;

            case 'disconnect':
	$this->_disconnect($_POST['reason']);
	break;

            case 'update':
	$this->update($_POST['last_version']);
	break;

            //unknown command
            default:
	echo 'error|Incorrect request.';
	seofb_log_die('Can\'t handle the POST request. ' . print_r($_POST, TRUE));
	break;
        }
    }

    /**
     * Receive a post from the Fightback server for posting in current blog
     */
    protected function _receive_fightback_post()
    {
        require_once(ABSPATH . 'wp-admin/includes/media.php');
        require_once(ABSPATH . 'wp-admin/includes/file.php');
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        $images = array();
        // upload image to wp media library & get their urls
        foreach ($_FILES as $key => $file)
        {
            $file_id = @media_handle_upload($key, 0);

            // check if error occured while uploading file
            if (is_wp_error($file_id))
            {
	$response = 'Image ' . $key . print_r($file, TRUE) . ' upload failed' . $file_id->get_error_message();
	seofb_log($response);
	echo 'error|' . $response;
	exit();
            }

            $images[$key] = wp_get_attachment_image($file_id, 'full');
        }
        // insert images urls to post content
        $content = str_replace(array_keys($images), $images, $_POST['post_content']);
        
        //insert videos - check if post has videos
        if(isset($_POST['videos']))
        {
            foreach(unserialize($_POST['videos']) as $shortcode => $url)
	$content = str_replace($shortcode, PHP_EOL . $url . PHP_EOL, $content);
        }
        
        //insert post to WP
        $wp_post_id = wp_insert_post(array(
            'post_title' => $_POST['post_title'],
            'post_content' => $content,
            'post_status' => 'publish',
            'post_category' => array($this->get_rand_category()),
            'tags_input' => $_POST['tag_list'],
            'post_author' => $this->get_rand_author_id(),
	));

        //insert post meta
        add_post_meta($wp_post_id, 'fightback_verify_code', $_POST['wp_post_verify_code']);

        if (FALSE !== $wp_post_id)
        {
            $response = 'success|' . $wp_post_id . '|' . get_permalink($wp_post_id);

            echo $response;
            seofb_log('Post was added. Response: ' . $response);
        }
        else
        {
            $response = 'error|' . $wp_post_id->get_error_message();

            echo $response;
            seofb_log('Error while post adding. Response: ' . $response);
        }

        exit();
    }

    /**
     * Delete a Fightback Post
     */
    protected function _delete_fightback_post()
    {
        //permanently delete a Post
        $result = wp_delete_post($_POST['wp_post_id'], TRUE);

        if (FALSE !== $result)
        {
            $response = 'success';

            echo $response;

            seofb_log('Post #' . $_POST['wp_post_id'] . ' was deleted. Response: ' . $response);
        }
        else
        {
            $response = 'error|Some WP error, can\' t delete a post #' . $_POST['wp_post_id'];

            echo $response;
            seofb_log('Error while post deleting. Response: ' . $response);
        }

        exit();
    }

    /**
     * Check current version of the plugin
     * Maybe need to autoupdate
     */
    function check_version()
    {
        // check if the current version is larger then the stored in 'fightback_last_version' option one
        if (!get_option('fightback_last_version') || (get_option('fightback_last_version') < SEOFB_PLUGIN_VERSION))
            update_option('fightback_last_version', SEOFB_PLUGIN_VERSION);

        $code = get_option('fightback_code');

        if (!$code)
        {
            return;
        }

        if (time() < get_option('fightback_next_version_checks_at'))
        {
            return;
        }

        $seofb_api = new SEOFB_WP_API($code);

        $response = $seofb_api->get('plugin_version', array('cur_version' => SEOFB_PLUGIN_VERSION));

        if ($response['data'] != NULL)
            update_option('fightback_last_version', $response['data']);

        //next checking after 1 hour
        update_option('fightback_next_version_checks_at', time() + 60 * 60);
    }

    public function get_categories_all($only_ids = FALSE)
    {
        $args = array(
            'type' => 'post',
            'orderby' => 'name',
            'order' => 'ASC',
            'hide_empty' => 0,
            'hierarchical' => 1,
            'taxonomy' => 'category',
        );
        $all_categories = get_categories($args);
        if ($only_ids)
        {
            $ids = array();
            foreach ($all_categories as $c)
            {
	$ids[] = $c->term_id;
            }
            return $ids;
        }
        return $all_categories;
    }

    public function get_rand_category()
    {
        $categories_to_post = get_option('fightback_categories') ? get_option('fightback_categories') : $this->get_categories_all(TRUE);
        $rand_cat_key = array_rand($categories_to_post, 1);
        return $categories_to_post[$rand_cat_key];
    }
    
    public function get_rand_author_id()
    {
        $seofb_authors = get_option('fightback_authors') ? get_option('fightback_authors') : get_users(array('who' => 'authors', 'fields' => 'ID'));
        
        return $seofb_authors[array_rand($seofb_authors)];
    }

    public function check_cat_qty()
    {
        $cat_qty = count($this->get_categories_all());
        // check if there are less then MIN_CATS_QTY=3 categories in blog
        if ($cat_qty < MIN_CATS_QTY)
        {
            $reason = 'More than ' . MIN_CATS_QTY . ' post categories must be there in your blog. Only ' . $cat_qty . ' of them found';
            $this->_disconnect_request(get_option('fightback_code'), $reason);
            $this->_disconnect($reason);
//            $this->_inspect_request('cat_qty');            
        }
    }

    public function theme_change()
    {
        $this->_inspect_request('switch_theme');
    }

    /**
     * Connect wp blog to FBN server - save the required options
     * @param string $fightback_code
     */
    protected function _connect($fightback_code = NULL)
    {
        if ($fightback_code)
            update_option('fightback_code', $fightback_code);

        update_option('fightback_status', array('status' => 'connected', 'date' => date('Y-m-d H:i')));

        // log the result
        seofb_log('WP Blog connected to FBN Server at ' . date('Y-m-d H:i'));
    }

    /**
     * Disconnect from FBN locally in blog
     * @param string $reason
     * @param boolean $clear_code TRUE if needed to clear the fightback code, FALSE otherwise
     */
    protected function _disconnect($reason, $clear_code = FALSE)
    {
        seofb_log('Disconnecting from FBN');
        update_option('fightback_status', array('status' => 'disconnected', 'date' => date('Y-m-d H:i'), 'reason' => is_array($reason) ? implode('<br />', $reason) : $reason));

        if ($clear_code)
            delete_option('fightback_code');

        // log the result
        seofb_log('WP Blog disconnected from FBN Server at ' . date('Y-m-d H:i'));
    }

    protected function _connect_request($fightback_code)
    {
        $seofb_connect = new SEOFB_WP_API($fightback_code);
        return $seofb_connect->post('connect', array
	    (
	    'blog_url' => get_option('siteurl'),
	    'wp_api_code' => $fightback_code
	        )
        );
    }

    protected function _disconnect_request($fightback_code, $reason)
    {
        $seofb_connect = new SEOFB_WP_API($fightback_code);
        return $seofb_connect->post('disconnect', array('blog_url' => get_option('siteurl'), 'reason' => $reason));
    }

    /**
     * Send request for inspection
     * @param string $reason Reason (cat_qty)
     */
    protected function _inspect_request($reason = NULL)
    {
        //send request to fightback server
        $seofb_connect = new SEOFB_WP_API(get_option('fightback_code'));
        $res = $seofb_connect->post('inspect', array('blog_url' => get_option('siteurl'), 'reason' => $reason));

        switch ($res['status'])
        {
            case 'error':
	$this->_disconnect($res['data']['reason']);
	break;

            case 'success':
	$this->_connect();

            default:
	break;
        }
    }

    /**
     * Plugin autoupdater
     * @return type
     */
    public function update($last_version)
    {
        include ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';
        include ABSPATH . 'wp-admin/includes/admin.php';

        // reset the check updates timeset
        add_filter('option__site_transient_update_plugins', 'clear_update_time');

        function clear_update_time($res)
        {
            $res->last_checked = 0;
            return $res;
        }
	
        // check for updates
        wp_update_plugins();
        
        // run updating
        $upgrader = new Plugin_Upgrader();
        $upgrader->upgrade(SEOFB_PLUGIN_NAME);

        // activate plugin after update
        activate_plugin(SEOFB_PLUGIN_NAME);

        // update option "current version"
        update_option('fightback_last_version', $last_version);
        exit();
    }
    
}