<?php

/**
 * SEO FightBack API to execute connection to FightBack server  
 */
Class SEOFB_WP_API
{

    protected $_fightbackCode;

    public function __construct($fightbackCode)
    {
        $this->_fightbackCode = $fightbackCode;
    }

    /**
     * Sends a POST request to FightBack server & gets the answer
     * 
     * @param string $command
     * @param array $data_to_post 
     * 
     * @return boolean|array FALSE if request failed, json-decoded data otherwise
     */
    public function post($command, array $data_to_post = array())
    {
        return $this->_request($command, 'post', $data_to_post);
    }

    /**
     * Sends a GET request to FightBack server & gets the answer
     * 
     * @param string $command
     * @param array $data_to_post 
     * 
     * @return boolean|array FALSE if request failed, json-decoded data otherwise
     */
    public function get($command, array $data_to_get = array())
    {
        return $this->_request($command, 'get', $data_to_get);
    }

    protected function _request($command, $httpMethod, array $data = array())
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_FAILONERROR, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);

        //different method to send data depending on HTTP method
        switch (strtolower($httpMethod))
        {
            case 'post':
	curl_setopt($ch, CURLOPT_URL, SEOFB_CONNECT_URL . $command . '/' . $this->_fightbackCode);
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
	break;

            case 'get':

	curl_setopt($ch, CURLOPT_URL, SEOFB_CONNECT_URL . $command . '/' . $this->_fightbackCode . '?' . http_build_query($data, '', '&'));
	break;
        }

        $output = curl_exec($ch);
        $result_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $curl_error = curl_errno($ch) . ': ' . curl_error($ch);
        curl_close($ch);

        //logging the response
        seofb_log("Connect response - HTTP Status Code: " . $result_code . "; " . $curl_error . ". Response: " . $output);

        // check if request is success
        if ($result_code == 200)
        {
            //if yes - return json-decoded data
            return json_decode($output, TRUE);
        }
        else
        {
            //if no - return FALSE
            return false;
        }
    }

}